/**
 * @since 1.0.0
 */
import * as HttpShardManager from "@effect/cluster/HttpShardManager";
import * as ShardingConfig from "@effect/cluster/ShardingConfig";
import * as ShardManager from "@effect/cluster/ShardManager";
import * as ShardStorage from "@effect/cluster/ShardStorage";
import * as SqlShardStorage from "@effect/cluster/SqlShardStorage";
import * as RpcSerialization from "@effect/rpc/RpcSerialization";
import * as Effect from "effect/Effect";
import * as Layer from "effect/Layer";
import { createServer } from "node:http";
import * as NodeHttpClient from "./NodeHttpClient.js";
import * as NodeHttpServer from "./NodeHttpServer.js";
import * as NodeSocket from "./NodeSocket.js";
/**
 * @since 1.0.0
 * @category Layers
 */
export const layerHttpServer = /*#__PURE__*/Effect.gen(function* () {
  const config = yield* ShardingConfig.ShardingConfig;
  return NodeHttpServer.layer(createServer, config.shardManagerAddress);
}).pipe(Layer.unwrapEffect);
/**
 * @since 1.0.0
 * @category Layers
 */
export const layer = options => {
  const layer = options.protocol === "http" ? HttpShardManager.layerHttp.pipe(Layer.provide([HttpShardManager.layerRunnerHealthHttp, layerHttpServer]), Layer.provide(NodeHttpClient.layerUndici)) : HttpShardManager.layerWebsocket.pipe(Layer.provide([HttpShardManager.layerRunnerHealthWebsocket, layerHttpServer]), Layer.provide(NodeSocket.layerWebSocketConstructor));
  return layer.pipe(Layer.provide(options?.storage === "sql" ? SqlShardStorage.layer : ShardStorage.layerNoop), Layer.provide([ShardingConfig.layerFromEnv(options.shardingConfig), ShardManager.layerConfigFromEnv(options?.config), options?.serialization === "ndjson" ? RpcSerialization.layerNdjson : RpcSerialization.layerMsgPack]));
};
//# sourceMappingURL=NodeClusterShardManagerHttp.js.map